(* Copyright 1991 by AT&T Bell Laboratories *)
(* Modified for Poly Rec by H.Leiss, cis.uni-muenchen.de *)

signature PPTYPELIST =
    sig	
	type ineq
	val ppineqlist : StaticEnv.staticEnv -> PrettyPrint.ppstream -> ineq list -> unit
	val pptypelist : StaticEnv.staticEnv -> PrettyPrint.ppstream -> Types.ty list -> unit
    end 

structure PPTypelist :PPTYPELIST =
struct

local open ErrorMsg Types List PPUtil 
in 
    structure PP = PrettyPrint

    type ineq = Semiunify.ineq
    val pps = PP.add_string
    fun en_pp ppstrm = {begin_block=PP.begin_block ppstrm, 
                        end_block=fn () => PP.end_block ppstrm, 
                        pps=PP.add_string ppstrm, 
                        add_break=PP.add_break ppstrm, 
                        add_newline=fn () => PP.add_newline ppstrm};

    val debugging = Control.debugging (* bool ref *)

    fun ppType env ppstrm ty = 
        if !debugging then
            ElabDebug.withInternals (fn () => PPType.ppType env ppstrm ty)
        else PPType.ppType env ppstrm ty

    fun ppineqlist1 env ppstrm L = 
        let val {pps,...} = en_pp ppstrm
        in 
            app (fn (Semiunify.ineq(Semiunify.mylab(_,lab),i,ty1,ty2)) =>
                 (PP.begin_block ppstrm PP.CONSISTENT 0;
                  pps "(";
                  ppType env ppstrm ty1;
                  pps "  <";pps (Int.toString i);pps " ";
                  pps (Symbol.name lab); pps "  "; 
                  ppType env ppstrm ty2; 
                  pps ")";
                  PP.add_newline ppstrm;
                  PP.end_block ppstrm)) L
        end

    and pptypelist1 env ppstrm (ty::rest) =
        let val {begin_block,end_block,pps,add_break,...} = en_pp ppstrm
        in 
            (ppType env ppstrm ty;
             pps ",  ";
             pptypelist1 env ppstrm rest) end
      | pptypelist1 env ppstrm [] = ()

    and ppineqlist (env:StaticEnv.staticEnv) ppstrm (ineqs:ineq list) : unit = 
        (PP.begin_block ppstrm PP.INCONSISTENT 1;
         ppineqlist1 env ppstrm ineqs;
         PP.end_block ppstrm)

    and pptypelist (env:StaticEnv.staticEnv) ppstrm (tylist:ty list) : unit = 
        (PP.begin_block ppstrm PP.INCONSISTENT 1;
         pptypelist1 env ppstrm tylist;
         PP.end_block ppstrm)

    val g = PrettyPrint.with_pp(ErrorMsg.defaultConsumer())


end
end (* structure PPTypelist *)





